#!/bin/bash
########################################################################
# build_pythia8.sh
########################################################################

####################################
# Useful variables
prog=${0##*/}
thisdir=$(cd "$(dirname "${0}")" && pwd -P)

####################################
# Useful functions.

# Report usage.
usage() {
  echo "USAGE: ${prog} <product_directory> <e19|e20|c7>:p382 debug|prof [tar]"
}

# Make sure we have UPS and ssibuildshims set up.
get_ssibuildshims() {
  [[ -n "${UPS_DIR}" ]] || { echo "ERROR: UPS not set up" 1>&2; exit 1; }
  source $("${UPS_DIR}/bin/ups" setup ${SETUP_UPS})
  setup ssibuildshims ${ssibuildshims_version} -z ${product_dir} || { \
    echo "ERROR: failed to set up ssibuildshims ${ssibuildshims_version}" 1>&2; \
    exit 1; }
}

# Handle qualifiers.
set_qualifiers() {
  local qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
  local qual
  for qual in "${qlist[@]}"; do
    case $qual in
      e19|e20|c7) cqual=$qual;;
      p382) pqual=$qual;;
      *) ssi_die "Unrecognized qualifier $qual"
    esac
  done
  [[ -n "$cqual" ]] || ssi_die "Base qualifier is mandatory"
  [[ -n "$pqual" ]] || ssi_die "A python qualifier is mandatory (e.g. p382)"
  basequal=${cqual}:${pqual}
}

# Set compiler flags.
set_compiler_flags() {
  case ${cqual} in
    e19|e20)
      cc=gcc
      cxx=g++
      cxxstd=17
      ;;
    c7)
      cc=clang
      cxx=clang++
      cxxstd=17
      ;;
    *)
      ssi_die "unrecognized compiler qualifier ${cqual}\n$(usage)"
  esac
  cxxflg+=( \
    -std=c++${cxxstd} -fPIC -pedantic \
    -W -Wall -Wshadow -Wno-deprecated-declarations \
    )
  case ${extraqual} in
    debug) cxxflg+=(-g -O0);;
    prof) cxxflg+=(-g -O3 -DNDEBUG -fno-omit-frame-pointer);;
    opt) cxxflg+=(-O3 -DNDEBUG);;
    *) ssi_die "please specify debug, opt, or prof\n$(usage)";;
  esac
}

# Make the distribution and leave if successful.
maybe_make_distribution() {
  [ "${maketar}" = "tar" ] && [ -d "${pkgdir}/lib" ] && \
    ups exist -j ${package} ${pkgver} -q+${fullqual//:/:+} \
    -z "${product_dir}" && \
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball \
    "${product_dir}" ${package} ${pkgver} ${fullqual} && \
    exit 0
  return $?
}

########################################################################
# Main.
########################################################################

####################################
# Arguments.
product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

# Basic check.
[[ -n "${product_dir}" ]] || \
  { printf "ERROR: please specify the local product directory\n$(usage)\n" 1>&2; \
  exit 1; }

####################################
# Package name and version information.
package=pythia8
origpkgver=v8_3_01
pkgver=${origpkgver}
ssibuildshims_version=v2_00_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtardir=${package%%[0-9]}${pkgdotver//./}
pkgtarfile=${pkgtardir}.tgz

# Obtain access to useful functions.
get_ssibuildshims
source ssi_functions

# Deal with qualifiers.
set_qualifiers

# Define globals.
# Need in-source build for now (blddir == sourcedir).
source define_basics IN_SOURCE

# Possible short-circuit.
maybe_make_distribution

echo "Building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

####################################
# Ensure directories.
mkdir -p "${pkgdir}" || \
  ssi_die "failed to create installation directory ${pkgdir}"
# Clean build (in-source).
rm -rf "${blddir}" && mkdir -p "${blddir}" || \
  ssi_die "failed to create build directory ${blddir}"
####################################

####################################
# Declare ourselves privately to allow setup of dependencies.
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product \
  "${product_dir}" ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q +${fullqual//:/:+} \
  -z "${fakedb}:${product_dir}:${PRODUCTS}" || \
  ssi_die "fake setup failed: missing dependency or qualifier not supported by table file?"
####################################

# Print active products.
echo
ups active
echo

# Ascertain compiler flags.
set_compiler_flags

cd "${sourcedir}" || ssi_die "unable to cd to source directory ${sourcedir}"

# Expand and patch the source.
bash -x prepare_source -P "${patchdir}" \
  "${tardir}/${pkgtarfile}" fix-lcg-detection.patch || exit $?

[ -n "${DEBUG_PATCHES}" ] && exit 1

# Ascertain and configure parallelism.
ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

# Current Pythia8 build system requires configure be invoked from the
# source (==build) directory.
cd "${blddir}/${pkgtardir}" \
  || ssi_die "unable to cd to build directory ${blddir}/${pkgtardir}"

####################################
# Configure, build, and install.

set -x

##################
# Configure.
#
# Notes:
#
# * --cxx-common overrides the default options (including -fPIC) so
#   we need to be very specific and complete.
#

# * HEPMC3 support is currently vestigial (configure only, not used in
#   build). Support is expected with 8.302, when we should
#   add:
#
#     --with-hepmc3-lib="${HEPMC3_LIB}" \
#
#   and uncomment / update the references in ups/pythia8.table as
#   appropriate.
"${sourcedir}/${pkgtardir}/configure" --prefix="${pkgdir}" \
  --cxx=${cxx} \
  --cxx-common="${cxxflg[*]}" \
  --with-gzip \
  --with-python-include="${PYTHON_INCLUDE}" \
  --with-lhapdf6="${LHAPDF_FQ_DIR}" \
  || ssi_die "configure failed"
##################

# Build.
make -j ${ncore} || ssi_die "make failed"

# Install.
make -j ${ncore} install || ssi_die "make install failed"

# Tidy source for installation as debug source.
make clean; find . -name '*~' | xargs rm -f

# Move cleaned source to installation point.
cd "${pkgups}/.." && mv "${blddir}" source || \
  ssi_die "unable to move source for packaging"

# Relocate the Python plugin to its final resting place.
python_plugin=${pkgdir}/lib/${package}.so
if [[ -f "${python_plugin}" ]]; then
  mkdir -p "${python_plugin%/*}/${PYTHON_LIBDIR}/site-packages/"
  mv -v "${python_plugin}" "${pkgdir}/${libdir}/" || \
    ssi_die "unable to move Python directory into ${pkgdir}/${libdir}"
fi

set +x

# Verify install.
[[ -d "${pkgdir}/lib" ]] || \
  ssi_die "failed to create ${pkgdir}/lib\n" \
   "      Something is very wrong"
####################################

# Declare the installed package.
ups exist ${package} ${pkgver} -z ${product_dir} || \
  ${SSIBUILDSHIMS_DIR}/bin/declare_product "${product_dir}" \
  ${package} ${pkgver} ${fullqual} || \
  ssi_die "Unable to declare product to UPS."

####################################
# Installation post-processing.

# Fix bin/pythia8-config.
pcfg="${pkgdir}/bin/pythia8-config"
mv "${pcfg}"{,\~}
cat "${pcfg}~" | \
  sed "s%${PYTHIA8_FQ_DIR}%\${PYTHIA8_FQ_DIR}%g" > "${pcfg}" && \
  chmod +x "${pcfg}" && \
  rm -f "${pcfg}~" || ssi_die "unable to fix pythia8-config"

# Fix Makefile.inc
${patchdir}/fix-makefile-inc "${pkgdir}/share/Pythia8/examples/Makefile.inc" || \
  ssi_die "unable to fix Makefile.inc"
####################################

echo "Finished building ${package} ${pkgver}"

# Attempt to set up the package.
setup -B ${package} ${pkgver} -q+${fullqual//:/:+} \
  -z "${product_dir}:${PRODUCTS}" || \
  ssi_die "unable to set up newly-built product."

echo "${package} is installed at ${pkgdir}"

# This must be last.
maybe_make_distribution
